]> git.sesse.net Git - backup.sh/blobdiff - backup.sh
Some minor changes that were done without checking out from RCS.
[backup.sh] / backup.sh
index a16ca3fb3e65512b35c587a585cab4f7f2da4a53..a76baf99d8432b6d2dcac06d77b95ebf6d4bf568 100755 (executable)
--- a/backup.sh
+++ b/backup.sh
@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 # backup.sh
 # 
 # Backup up Unix-like computers
 # mind when hacking:
 #
 # KISS. And by that I mean really simple. Only regular Unix commands, please
-# (but GNU extensions are used).
+# (but GNU extensions are used and moreutils has to be installed).
 #
 # Because of the way we run this script from cron, informational logging goes
 # to stderr and warnings and errors to stdout. This way, we get mail whenever
 # stuff fails, and other output is redirected to the log file.
 # 
-# The computers are backed up in parallell (but the filesystems on each
+# The computers are backed up in parallel (but the filesystems on each
 # computer in serial). This is implemented highly simplistic, with forking
 # subshells. The script has no concept about threads, and stuff might go wrong,
 # which usually requires manual cleanups.
 #
 # The script is halfway Norwegian and halfway English. Newer modifications are
 # in English, we should probably stick to that.
-
-# TODO(sesse, 2007-12-16): 
-#  * Sjekk hvorfor .idx-filene til UD er blitt 0 for siste fullbackuper
-#  * Rydde uleselig kode fra linje 211 og litt nedover
+# 
+# Known issues:
+#
+# o $R has no proper meaning because of the forking subshell stuff. We need a
+#   more robust way of returning non-zero on errors, which we probably don't
+#   right now.
+#
 
 # Configuration
 LOCKFILE=/home/backup/backuprun.lockfile
-confdir=/home/backup/conf/           # Configuration files
-storagedir=/backup                   # Where we keep backups
-maksantallfulle=3                    # Number of full backups
-dagermellomfulle=30                  # Days between full backups
-DATE=`date "+%Y%m%d%H%M"`            #format: touch
-DATEs=`date "+%Y-%m-%d %H:%M"`       #format: tar
+confdir=/home/backup/conf/              # Configuration files
+storagedir=/backup                      # Where we keep backups
+maksantallfulle=3                       # Number of full backups
+dagermellomfulle=30                     # Days between full backups
+dagermellomfulleforfast=32              # Days between full backups for machines with fixed
+                                        # full-backup date (in case the fixed day gets missed)
+DAY_OF_MONTH=`date "+%d" | sed s/^0//`  # from 1 to 31
+DATE=`date "+%Y%m%d%H%M"`               #format: touch
+DATEs=`date "+%Y-%m-%d %H:%M"`          #format: tar
 
 # Exclude-pattern
 exclude=$confdir/exclude
@@ -59,11 +65,12 @@ trap die SIGINT SIGTERM
 
 # Don't start if we're already running
 if [ -e $LOCKFILE ]; then
-       echo `date`": $LOCKFILE exists, exiting."
-       echo `date`": $LOCKFILE exists, exiting." >&2
+       echo `date`": $LOCKFILE exists for pid `cat $LOCKFILE`, exiting."
+       echo `date`": $LOCKFILE exists for pid `cat $LOCKFILE`, exiting." >&2
        exit 1
 fi;
 touch $LOCKFILE
+echo $$ > $LOCKFILE
 
 #syntax of remotestat:
 #hostname:/directory/
@@ -96,7 +103,7 @@ backup()
 {
 
  echo -n `date` >&2
- printf " $computer: $computer:$filesystem $backuplevel backup" >&2
+ printf " $computer: $computer:$filesystem $backuplevel backup\n" >&2
 
  if [ "$backuplevel" = "daglig" ] || [ "$backuplevel" = "incremental" ]
  then
@@ -106,14 +113,6 @@ backup()
   lastcmd=""
  fi
 
- #if this client has a special tar
- #we need to find a better solution to this conf-issue
- if [ -f $confdir/tar.$computer ] ; then
-   tar=`cat $confdir/tar.$computer`
- else 
-   tar=tar
- fi
-
  # expand the exclude-path for use with tar
  exf=`ssh root@$computer "ls ~/.backup/exclude"`
 
@@ -128,8 +127,8 @@ backup()
  #    Pipe the stuff over ssh to ourselves, run pee to cat the contents to a
  #    file and build a file list, respectivly.  
  TARFILE=$DATE.tmp
- TARCMD="ssh root@$computer \"$tar --one-file-system -zcf - -C $filesystem . $lastcmd \
-       --exclude-from=$exf\" | pee \"cat > $TARFILE\" \"tar tvf -\""
+ TARCMD="ssh root@$computer \"nice -n 19 ionice -c3 tar --one-file-system -zcf - -C $filesystem . $lastcmd \
+       --exclude-from=$exf\" | pee \"cat > $TARFILE\" \"tar tzvf -\""
  echo `date`" $computer: Running $TARCMD" >&2
  eval $TARCMD > $DATE.idx
 
@@ -142,8 +141,8 @@ backup()
   # Update timestamp
   echo $DATEs > ../.date && touch -t $DATE ../.date
 
-  # Make a sorted filelist
-  grep -v ^d $DATE.idx | sort -n -r -k 2 > $DATE.sdx
+  # Make a sorted filelist as well
+  sort -k6 < $DATE.idx > $DATE.sdx
 
   # Fix permissions
   chmod 600 *tgz                      #only for us
@@ -208,21 +207,20 @@ do
  fi
 
  # Try to copy preeexec and postexec if they exist
-# TODO: Gah, clean this mess!
- [ -f $confdir/preexec.$computer ] && (
-    scp $confdir/preexec.$computer  root@$computer:~/.backup/preexec >&2 ||
-     ( echo `date`" $computer: Could not copy preexec.$computer to $computer:~/.backup/preexec - skipping backup of $computer"
-       R=1
-       continue
-     )
-    )
- [ -f $confdir/postexec.$computer ] && (
-    scp $confdir/postexec.$computer root@$computer:~/.backup/postexec >&2 ||
-     ( echo `date`" $computer: Could not copy postexec.$computer to $computer:~/.backup/postexec - skipping backup of $computer"
-       R=1
-       continue
-     )
-    )
+ if [ -f $confdir/preexec.$computer ]; then
+   if ! scp $confdir/preexec.$computer  root@$computer:~/.backup/preexec >&2; then
+     echo `date`" $computer: Could not copy preexec.$computer to $computer:~/.backup/preexec - skipping backup of $computer"
+     R=1
+     continue
+   fi
+ fi
+ if [ -f $confdir/postexec.$computer ]; then
+   if ! scp $confdir/postexec.$computer root@$computer:~/.backup/postexec >&2; then
+     echo `date`" $computer: Could not copy postexec.$computer to $computer:~/.backup/postexec - skipping backup of $computer"
+     R=1
+     continue
+   fi
+ fi
 
  # Try to run preexec if it exist
  if ! ssh root@$computer "[ ! -f ~/.backup/preexec ] || /bin/bash -x ~/.backup/preexec" >&2; then
@@ -252,11 +250,26 @@ do
    echo $DATEs > $storagedir/$computer/$sfilesystem/.date
   fi
 
-  # Check if we want a full backup
-  if [ -z "`find $storagedir/$computer/$sfilesystem/full/ -name \*tgz -mtime -$dagermellomfulle`" ]; then
+  # Check if we want a full backup - Debug statement that causes noise in cron mail:
+  # printf " $computer $filesystem: $confdir/fastfullbackupdag.$computer.$sfilesystem\n"
+  if [ -f $confdir/fastfullbackupdag.$computer.$sfilesystem ]; then
+    fullbackup_min_for_this_machine=$dagermellomfulleforfast
+    if [ "$DAY_OF_MONTH" = "`cat $confdir/fastfullbackupdag.$computer.$sfilesystem`" ]; then
+      backuplevel=full
+    fi
+  elif [ -f $confdir/fastfullbackupdag.$computer ]; then
+    fullbackup_min_for_this_machine=$dagermellomfulleforfast
+    if [ "$DAY_OF_MONTH" = "`cat $confdir/fastfullbackupdag.$computer`" ]; then
+      backuplevel=full
+    fi
+  else
+    fullbackup_min_for_this_machine=$dagermellomfulle 
+  fi
+
+  if [ -z "`find $storagedir/$computer/$sfilesystem/full/ -name \*tgz -mtime -$fullbackup_min_for_this_machine`" ]; then
    backuplevel=full
   fi
+
   # We want to be here
   cd $storagedir/$computer/$sfilesystem/$backuplevel || die